from .config import Config, ctypes
from .config import auto_encode

# INT64 __stdcall HCVMDMA_GetVersion(int windowsIndex);
def HDDMA_获取版本号(窗口序号: int) -> int:
    """
    获取DMA服务器版本号
    :param 窗口序号: 窗口序号
    :return: 返回长整数版本号，如240401 格式:24年-04版本号-01小版本号
    """
    hd = Config.get_hd()

    HCVMDMA_GetVersion = hd.HCVMDMA_GetVersion
    HCVMDMA_GetVersion.restype = ctypes.c_int64
    HCVMDMA_GetVersion.argtypes = [ctypes.c_int32]

    return HCVMDMA_GetVersion(ctypes.c_int32(窗口序号))

# INT64 __stdcall HCVMDMA_IsVersion(int windowsIndex);
def HDDMA_版本是否一致(窗口序号: int) -> int:
    """
    判断HD插件的DMA接口和DMA服务器接口版本是否一致
    :param 窗口序号: 窗口序号
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    HCVMDMA_IsVersion = hd.HCVMDMA_IsVersion
    HCVMDMA_IsVersion.restype = ctypes.c_int64
    HCVMDMA_IsVersion.argtypes = [ctypes.c_int32]

    return HCVMDMA_IsVersion(ctypes.c_int32(窗口序号))

# INT64 __stdcall HCVMDMA_StartServer(char* ip = "0.0.0.0", int port = 6532, char* serverRootPath = "");
def HDDMA_开启服务器(ip: str = "127.0.0.1", port: int = 6532, 服务器exe根目录: str = "") -> int:
    """
    开启DMA服务器
    :param ip: IP地址 (默认127.0.0.1)
    :param port: 端口 (默认6532)
    :param 服务器exe根目录: 兼容指定自定义进程名(全路径\\XXXX.exe) 不指定路径使用内置进程名
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    HCVMDMA_StartServer = hd.HCVMDMA_StartServer
    HCVMDMA_StartServer.restype = ctypes.c_int64
    HCVMDMA_StartServer.argtypes = [ctypes.c_char_p, ctypes.c_int32, ctypes.c_char_p]

    ip_bytes = auto_encode(ip)
    path_bytes = auto_encode(服务器exe根目录)

    return HCVMDMA_StartServer(ip_bytes, ctypes.c_int32(port), path_bytes)

# INT64 __stdcall HCVMDMA_CloseServer();
def HDDMA_关闭服务器() -> int:
    """
    关闭服务器,同时结束进程
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    HCVMDMA_CloseServer = hd.HCVMDMA_CloseServer
    HCVMDMA_CloseServer.restype = ctypes.c_int64

    return HCVMDMA_CloseServer()

# INT64 __stdcall HCVMDMA_ServerIsStart();
def HDDMA_服务器是否开启(ip: str = "127.0.0.1", port: int = 6532) -> int:
    """
    接口说明:
        服务器是否已经开启

    函数原型：
        __int64  __stdcall  HCVMDMA_ServerIsStart(char* ip = "127.0.0.1", int port = 6532);

    参数说明:
        ip: 指定服务器IP,指定后可以用来判断服务器是否开启
        port: 指定服务器端口,指定后可以用来判断服务器是否开启

    返回值:
        int: 见HD返回值表

    备注:
        循环检查直到服务器开启成功
        如果使用HCVMDMA_StartServer开启服务器,不指定IP和端口,则使用的是HCVMDMA_ServerIsStart指定的IP和端口
    """
    hd = Config.get_hd()
    HCVMDMA_ServerIsStart = hd.HCVMDMA_ServerIsStart
    HCVMDMA_ServerIsStart.restype = ctypes.c_int64
    HCVMDMA_ServerIsStart.argtypes = [ctypes.c_char_p, ctypes.c_int32]
    ip_bytes = auto_encode(ip)
    ret = HCVMDMA_ServerIsStart(ip_bytes, ctypes.c_int32(port))
    return ret

# INT64 __stdcall HCVMDMA_Init(int windowsIndex, int vmPid, int timeOut = 10000);
def HDDMA_关联虚拟机(窗口序号: int, 虚拟机PID: int, 超时等待时间: int = 10000) -> int:
    """
    初始化并关联虚拟机
    :param 窗口序号: 窗口序号
    :param 虚拟机PID: 虚拟机进程ID (进程名: vmware-vmx.exe)
    :param 超时等待时间: 超时等待时间 (默认10000毫秒)
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    HCVMDMA_Init = hd.HCVMDMA_Init
    HCVMDMA_Init.restype = ctypes.c_int64
    HCVMDMA_Init.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int32]

    return HCVMDMA_Init(ctypes.c_int32(窗口序号), ctypes.c_int32(虚拟机PID), ctypes.c_int32(超时等待时间))

# INT64  __stdcall  HCVMDMA_GetPid(int windowsIndex, char* processName);
def HDDMA_获取PID(窗口序号: int, 进程名: str) -> int:
    """
    通过进程名获取PID
    :param 窗口序号: 窗口序号 (从1开始)
    :param 进程名: 进程名
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_GetPid = hd.HCVMDMA_GetPid
    HCVMDMA_GetPid.restype = ctypes.c_int64
    HCVMDMA_GetPid.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_char_p  # 进程名
    ]

    # 将进程名转换为字节串
    process_name_bytes = auto_encode(进程名)

    # 调用C函数
    ret = HCVMDMA_GetPid(
        ctypes.c_int32(窗口序号),
        process_name_bytes
    )

    return ret

# INT64  __stdcall  HCVMDMA_Read(int windowsIndex, int targetPid, INT64 addr, int addrSzie);
def HDDMA_读地址(窗口序号: int, 目标进程PID: int, 地址: int, 地址大小: int) -> int:
    """
    读地址,仅支持 1 2 4 8 地址大小
    :param 窗口序号: 窗口序号 (从1开始)
    :param 目标进程PID: 虚拟机中的目标进程PID
    :param 地址: 读的地址
    :param 地址大小: 读的大小,仅支持 1 2 4 8 地址大小
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_Read = hd.HCVMDMA_Read
    HCVMDMA_Read.restype = ctypes.c_int64
    HCVMDMA_Read.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 目标进程PID
        ctypes.c_int64,  # 地址
        ctypes.c_int32   # 地址大小
    ]

    # 调用C函数
    ret = HCVMDMA_Read(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_int64(地址),
        ctypes.c_int32(地址大小)
    )

    return ret

# INT64  __stdcall  HCVMDMA_ReadFloat(int windowsIndex, int targetPid, INT64 addr);
def HDDMA_读单浮点(窗口序号: int, 目标进程PID: int, 地址: int) -> int:
    """
    读单浮点数
    :param 窗口序号: 窗口序号 (从1开始)
    :param 目标进程PID: 虚拟机中的目标进程PID
    :param 地址: 读的地址
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_ReadFloat = hd.HCVMDMA_ReadFloat
    HCVMDMA_ReadFloat.restype = ctypes.c_int64
    HCVMDMA_ReadFloat.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 目标进程PID
        ctypes.c_int64   # 地址
    ]

    # 调用C函数
    ret = HCVMDMA_ReadFloat(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_int64(地址)
    )

    return ret

# INT64  __stdcall  HCVMDMA_ReadDouble(int windowsIndex, int targetPid, INT64 addr);
def HDDMA_读双浮点(窗口序号: int, 目标进程PID: int, 地址: int) -> int:
    """
    读双浮点数
    :param 窗口序号: 窗口序号 (从1开始)
    :param 目标进程PID: 虚拟机中的目标进程PID
    :param 地址: 读的地址
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_ReadDouble = hd.HCVMDMA_ReadDouble
    HCVMDMA_ReadDouble.restype = ctypes.c_int64
    HCVMDMA_ReadDouble.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 目标进程PID
        ctypes.c_int64   # 地址
    ]

    # 调用C函数
    ret = HCVMDMA_ReadDouble(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_int64(地址)
    )

    return ret

# INT64  __stdcall  HCVMDMA_ReadBytes(int windowsIndex, int targetPid, INT64 addr, BYTE* buffAddr, int readSzie);
def HDDMA_读字节集(窗口序号: int, 目标进程PID: int, 地址: int, 缓冲区地址, 读字节大小: int) -> int:
    """
    读字节集
    :param 窗口序号: 窗口序号 (从1开始)
    :param 目标进程PID: 虚拟机中的目标进程PID
    :param 地址: 读的地址
    :param 缓冲区地址: 传递一个用于存字节集的缓冲区,一定要大于等于readSzie
    :param 读字节大小: 读的大小,内部限制最大1024字节,不能超过1024
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_ReadBytes = hd.HCVMDMA_ReadBytes
    HCVMDMA_ReadBytes.restype = ctypes.c_int64
    HCVMDMA_ReadBytes.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 目标进程PID
        ctypes.c_int64,  # 地址
        ctypes.POINTER(ctypes.c_ubyte),  # 缓冲区地址
        ctypes.c_int32   # 读字节大小
    ]

    # 调用C函数
    ret = HCVMDMA_ReadBytes(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_int64(地址),
        缓冲区地址,
        ctypes.c_int32(读字节大小)
    )

    return ret

# INT64  __stdcall  HCVMDMA_Write(int windowsIndex, int targetPid, INT64 value, INT64 addr, int addrSzie);
def HDDMA_写地址(窗口序号: int, 目标进程PID: int, 值: int, 地址: int, 地址大小: int) -> int:
    """
    写地址,仅支持 1 2 4 8 地址大小
    :param 窗口序号: 窗口序号 (从1开始)
    :param 目标进程PID: 虚拟机中的目标进程PID
    :param 值: 待写入的值
    :param 地址: 读的地址
    :param 地址大小: 读的大小,仅支持 1 2 4 8 地址大小
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_Write = hd.HCVMDMA_Write
    HCVMDMA_Write.restype = ctypes.c_int64
    HCVMDMA_Write.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 目标进程PID
        ctypes.c_int64,  # 值
        ctypes.c_int64,  # 地址
        ctypes.c_int32   # 地址大小
    ]

    # 调用C函数
    ret = HCVMDMA_Write(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_int64(值),
        ctypes.c_int64(地址),
        ctypes.c_int32(地址大小)
    )

    return ret

# INT64 __stdcall HCVMDMA_WriteFloat(int windowsIndex, int targetPid, float value, INT64 addr);
def HDDMA_写单浮点(窗口序号: int, 目标进程PID: int, 单浮点数值: float, 地址: int) -> int:
    """
    写单浮点数，需要先初始化关联虚拟机HCVMDMA_Init
    
    Args:
        窗口序号: 窗口序号
        目标进程PID: 虚拟机中的目标进程PID
        单浮点数值: 要写入的单浮点数值
        地址: 要写入的地址
    
    Returns:
        int: 返回值代码（查看HD返回值表）
    """
    hd = Config.get_hd()
    
    HCVMDMA_WriteFloat = hd.HCVMDMA_WriteFloat
    HCVMDMA_WriteFloat.restype = ctypes.c_int64
    HCVMDMA_WriteFloat.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_float, ctypes.c_int64]
    
    ret = HCVMDMA_WriteFloat(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_float(单浮点数值),
        ctypes.c_int64(地址)
    )
    return ret

# INT64 __stdcall HCVMDMA_WriteDouble(int windowsIndex, int targetPid, double value, INT64 addr);
def HDDMA_写双浮点(窗口序号: int, 目标进程PID: int, 双浮点数值: float, 地址: int) -> int:
    """
    写双浮点数，需要先初始化关联虚拟机HCVMDMA_Init
    
    Args:
        窗口序号: 窗口序号
        目标进程PID: 虚拟机中的目标进程PID
        双浮点数值: 要写入的双浮点数值
        地址: 要写入的地址
    
    Returns:
        int: 返回值代码（查看HD返回值表）
    """
    hd = Config.get_hd()
    
    HCVMDMA_WriteDouble = hd.HCVMDMA_WriteDouble
    HCVMDMA_WriteDouble.restype = ctypes.c_int64
    HCVMDMA_WriteDouble.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_double, ctypes.c_int64]
    
    ret = HCVMDMA_WriteDouble(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_double(双浮点数值),
        ctypes.c_int64(地址)
    )
    return ret

# INT64 __stdcall HCVMDMA_WriteBytes(int windowsIndex, int targetPid, INT64 addr, BYTE* buffAddr, int writeSize);
def HDDMA_写字节集(窗口序号: int, 目标进程PID: int, 地址: int, 缓冲区地址: int, 写字节大小: int) -> int:
    """
    写字节集，需要先初始化关联虚拟机HCVMDMA_Init
    
    Args:
        窗口序号: 窗口序号
        目标进程PID: 虚拟机中的目标进程PID
        地址: 目标进程中要写的地址
        缓冲区地址: 存储字节集的缓冲区地址
        写字节大小: 写入的字节大小，最大值为1024
    
    Returns:
        int: 返回值代码（查看HD返回值表）
    """
    hd = Config.get_hd()
    
    HCVMDMA_WriteBytes = hd.HCVMDMA_WriteBytes
    HCVMDMA_WriteBytes.restype = ctypes.c_int64
    HCVMDMA_WriteBytes.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int64, ctypes.c_void_p, ctypes.c_int32]
    
    ret = HCVMDMA_WriteBytes(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(目标进程PID),
        ctypes.c_int64(地址),
        ctypes.c_void_p(缓冲区地址),
        ctypes.c_int32(写字节大小)
    )
    return ret

# INT64 __stdcall HCVMDMA_GetProcAddr(int windowsIndex, int targetPid, char* moduleName, char* procName);
def HDDMA_获取函数(窗口序号: int, 目标进程PID: int, 模块名字: str, 函数名字: str) -> int:
    """
    获取函数地址 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 目标进程PID
    :param 模块名字: 模块名字
    :param 函数名字: 函数名字
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_GetProcAddr = hd.HCVMDMA_GetProcAddr
    HCVMDMA_GetProcAddr.restype = ctypes.c_int64
    HCVMDMA_GetProcAddr.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 目标进程PID
        ctypes.c_char_p,  # 模块名字
        ctypes.c_char_p   # 函数名字
    ]

    # 将模块名字和函数名字转换为字节串
    module_bytes = auto_encode(模块名字)
    proc_bytes = auto_encode(函数名字)

    # 调用C函数
    ret = HCVMDMA_GetProcAddr(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_char_p(module_bytes),
        ctypes.c_char_p(proc_bytes)
    )
    return ret

# INT64  __stdcall  HCVMDMA_GetBaseModule(int windowsIndex, int targetPid);
def HDDMA_获取主模块(窗口序号, 虚拟机中的目标进程PID):
    """
    获取主模块地址 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 目标进程PID
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_GetBaseModule = hd.HCVMDMA_GetBaseModule
    HCVMDMA_GetBaseModule.restype = ctypes.c_int64
    HCVMDMA_GetBaseModule.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32    # 目标进程PID
    ]

    # 调用C函数
    ret = HCVMDMA_GetBaseModule(ctypes.c_int32(窗口序号), ctypes.c_int32(虚拟机中的目标进程PID))
    return ret

# INT64  __stdcall  HCVMDMA_GetModule(int windowsIndex, int targetPid, char* moduleName);
def HDDMA_获取模块(窗口序号, 虚拟机中的目标进程PID, 模块名字):
    """
    获取模块地址 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 目标进程PID
    :param 模块名字: 模块名字，如ntdll.dll
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_GetModule = hd.HCVMDMA_GetModule
    HCVMDMA_GetModule.restype = ctypes.c_int64
    HCVMDMA_GetModule.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 目标进程PID
        ctypes.c_char_p   # 模块名字
    ]

    # 将模块名字转换为字节串
    module_bytes = auto_encode(模块名字)

    # 调用C函数
    ret = HCVMDMA_GetModule(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_char_p(module_bytes)
    )
    return ret

# INT64  __stdcall HCVMDMA_ShellCodeInitX64(int windowsIndex, int targetPid, INT64 hookAddr, int hookAddrSzie);
def HDDMA_获取执行环境X64(窗口序号, 虚拟机中的目标进程PID, HOOK地址, 破坏的汇编字节数):
    """
    初始化获取一个某个进程(x64)的执行环境句柄
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param HOOK地址: HOOK地址 (如:一些进程频繁用到的函数 sleep closehandle deletedc 等等)
    :param 破坏的汇编字节数: 破坏的汇编字节数, 必须是完整的汇编, 不能被打断
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_ShellCodeInitX64 = hd.HCVMDMA_ShellCodeInitX64
    HCVMDMA_ShellCodeInitX64.restype = ctypes.c_int64
    HCVMDMA_ShellCodeInitX64.argtypes = [
        ctypes.c_int32,    # 窗口序号
        ctypes.c_int32,    # 虚拟机中的目标进程PID
        ctypes.c_int64,    # HOOK地址
        ctypes.c_int32     # 破坏的汇编字节数
    ]

    # 调用C函数
    ret = HCVMDMA_ShellCodeInitX64(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_int64(HOOK地址),
        ctypes.c_int32(破坏的汇编字节数)
    )
    return ret

# INT64  __stdcall HCVMDMA_RunShellCodeX64(int windowsIndex, int targetPid, INT64 excuteEnvAddr, char* shellCodeStr);
def HDDMA_CALLX64(窗口序号, 虚拟机中的目标进程PID, 执行环境句柄, shellCode机器码):
    """
    通过指定一个初进程(x64)的执行环境句柄来执行CALL
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param 执行环境句柄: 执行环境句柄, 通过HCVMDMA_ShellCodeInitX64 获取
    :param shellCode机器码: 自己写的汇编shellcode机器码, 最大限制不超过1024字节
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_RunShellCodeX64 = hd.HCVMDMA_RunShellCodeX64
    HCVMDMA_RunShellCodeX64.restype = ctypes.c_int64
    HCVMDMA_RunShellCodeX64.argtypes = [
        ctypes.c_int32,    # 窗口序号
        ctypes.c_int32,    # 虚拟机中的目标进程PID
        ctypes.c_int64,    # 执行环境句柄
        ctypes.c_char_p   # shellCode机器码
    ]

    # 将shellCode机器码转换为字节串
    shell_code_bytes = auto_encode(shellCode机器码)

    # 调用C函数
    ret = HCVMDMA_RunShellCodeX64(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_int64(执行环境句柄),
        ctypes.c_char_p(shell_code_bytes)
    )
    return ret

# INT64  __stdcall HCVMDMA_ShellCodeInitx86(int windowsIndex, int targetPid, INT64 hookAddr, int hookAddrSize);
def HDDMA_获取执行环境x86(窗口序号, 虚拟机中的目标进程PID, HOOK地址, HOOK字节数):
    """
    初始化获取一个某个进程(x86)的执行环境句柄
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param HOOK地址: HOOK地址，例如一些进程频繁用到的函数，如sleep, closehandle, deletedc等
    :param HOOK字节数: 破坏的汇编字节数，必须是完整的汇编，不能被打断
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_ShellCodeInitx86 = hd.HCVMDMA_ShellCodeInitx86
    HCVMDMA_ShellCodeInitx86.restype = ctypes.c_int64
    HCVMDMA_ShellCodeInitx86.argtypes = [
        ctypes.c_int32,    # 窗口序号
        ctypes.c_int32,    # 虚拟机中的目标进程PID
        ctypes.c_int64,    # HOOK地址
        ctypes.c_int32     # HOOK字节数
    ]

    # 调用C函数
    ret = HCVMDMA_ShellCodeInitx86(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_int64(HOOK地址),
        ctypes.c_int32(HOOK字节数)
    )
    return ret

# INT64  __stdcall HCVMDMA_RunShellCodex86(int windowsIndex, int targetPid, INT64 excuteEnvAddr, char* shellCodeStr);
def HDDMA_CALLx86(窗口序号, 虚拟机中的目标进程PID, 执行环境句柄, shellCode机器码):
    """
    通过指定一个初进程(x86)的执行环境句柄来执行CALL
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param 执行环境句柄: 执行环境句柄, 通过HCVMDMA_ShellCodeInitx86 获取
    :param shellCode机器码: 自己写的汇编shellcode机器码, 最大限制不超过1024字节
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_RunShellCodex86 = hd.HCVMDMA_RunShellCodex86
    HCVMDMA_RunShellCodex86.restype = ctypes.c_int64
    HCVMDMA_RunShellCodex86.argtypes = [
        ctypes.c_int32,    # 窗口序号
        ctypes.c_int32,    # 虚拟机中的目标进程PID
        ctypes.c_int64,    # 执行环境句柄
        ctypes.c_char_p   # shellCode机器码
    ]

    # 将shellCode机器码转换为字节串
    shell_code_bytes = auto_encode(shellCode机器码)

    # 调用C函数
    ret = HCVMDMA_RunShellCodex86(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_int64(执行环境句柄),
        ctypes.c_char_p(shell_code_bytes)
    )
    return ret

# INT64  __stdcall HCVMDMA_FindCodeX86(int windowsIndex, int targetPid, CHAR* code, int offset, int num, int type = 1, CHAR* moduleName = NULL);
def HDDMA_搜索特征码x86(窗口序号, 虚拟机中的目标进程PID, 特征码字符串地址, 偏移, 次数, 类型=1, 模块名字=None):
    """
    特征码搜索(X86)
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param 特征码字符串地址: 特征码字符串地址 (支持??)
    :param 偏移: 正负偏移
    :param 次数: 找到的第几次 从1开始
    :param 类型: 获取的是(基地址/CALL地址/直接地址/地址中的值) 1为地址 2为基地址 3为CALL 4为地址中的值
    :param 模块名字: 在指定的模块中搜索, 默认为主模块 NULL为默认
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_FindCodeX86 = hd.HCVMDMA_FindCodeX86
    HCVMDMA_FindCodeX86.restype = ctypes.c_int64
    HCVMDMA_FindCodeX86.argtypes = [
        ctypes.c_int32,    # 窗口序号
        ctypes.c_int32,    # 虚拟机中的目标进程PID
        ctypes.c_char_p,   # 特征码字符串地址
        ctypes.c_int32,    # 偏移
        ctypes.c_int32,    # 次数
        ctypes.c_int32,    # 类型
        ctypes.c_char_p    # 模块名字
    ]

    # 将特征码字符串地址和模块名字转换为字节串
    code_bytes = auto_encode(特征码字符串地址)
    module_name_bytes = auto_encode(模块名字 or '')

    # 调用C函数
    ret = HCVMDMA_FindCodeX86(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_char_p(code_bytes),
        ctypes.c_int32(偏移),
        ctypes.c_int32(次数),
        ctypes.c_int32(类型),
        ctypes.c_char_p(module_name_bytes)
    )
    return ret

# INT64  __stdcall   HCVMDMA_FindCodeX86Ex(int windowsIndex, int targetPid, INT64 beginAddr, int addrSize, CHAR* code, int offset, int num, int type = 1);
def HDDMA_搜索特征码x86Ex(窗口序号, 虚拟机中的目标进程PID, 开始地址, 搜索范围大小, 特征码字符串地址, 正负偏移, 次数, 类型=1):
    """
    特征码搜索(X86)(扩展版本) 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 目标进程PID
    :param 开始地址: 搜索的开始地址
    :param 搜索范围大小: 搜索范围的大小，单位是字节
    :param 特征码字符串地址: 特征码字符串地址，支持??，可以是十六进制
    :param 正负偏移: 偏移量，支持正负数
    :param 次数: 找到的第几次，从1开始
    :param 类型: 需要获取的类型，1为地址，2为基地址，3为CALL地址，4为地址中的值（默认1）
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_FindCodeX86Ex = hd.HCVMDMA_FindCodeX86Ex
    HCVMDMA_FindCodeX86Ex.restype = ctypes.c_int64
    HCVMDMA_FindCodeX86Ex.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 目标进程PID
        ctypes.c_int64,   # 开始地址
        ctypes.c_int32,   # 搜索范围大小
        ctypes.c_char_p,  # 特征码字符串地址
        ctypes.c_int32,   # 偏移量
        ctypes.c_int32,   # 次数
        ctypes.c_int32    # 类型
    ]

    # 将特征码字符串转换为字节串
    code_bytes = auto_encode(特征码字符串地址)

    # 调用C函数
    ret = HCVMDMA_FindCodeX86Ex(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_int64(开始地址),
        ctypes.c_int32(搜索范围大小),
        ctypes.c_char_p(code_bytes),
        ctypes.c_int32(正负偏移),
        ctypes.c_int32(次数),
        ctypes.c_int32(类型)
    )
    return ret

# INT64 __stdcall HCVMDMA_FindCodeX64(int windowsIndex, int targetPid, CHAR* code, int offset, int num, int type = 1, CHAR* moduleName = NULL)
def HDDMA_搜索特征码x64(窗口序号, 虚拟机中的目标进程PID, 特征码字符串, 偏移, 次数, 类型=1, 模块名字=None):
    """
    特征码搜索 (X64)

    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param 特征码字符串: 特征码字符串地址(支持??)
    :param 偏移: 正负偏移
    :param 次数: 找到的第几次，从1开始
    :param 类型: 获取的是(基地址/CALL地址/直接地址/地址中的值) 1为地址，2为基地址，3为CALL，4为地址中的值
    :param 模块名字: 在指定的模块中搜索, 默认为主模块。NULL为默认
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_FindCodeX64 = hd.HCVMDMA_FindCodeX64
    HCVMDMA_FindCodeX64.restype = ctypes.c_int64
    HCVMDMA_FindCodeX64.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 虚拟机中的目标进程PID
        ctypes.c_char_p,  # 特征码字符串
        ctypes.c_int32,  # 偏移
        ctypes.c_int32,  # 次数
        ctypes.c_int32,  # 类型
        ctypes.c_char_p  # 模块名字 (可选，默认 NULL)
    ]

    # 将特征码字符串和模块名字转换为字节串
    code_bytes = auto_encode(特征码字符串)
    module_name_bytes = auto_encode(模块名字) if 模块名字 else None

    # 调用C函数
    ret = HCVMDMA_FindCodeX64(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        code_bytes,
        ctypes.c_int32(偏移),
        ctypes.c_int32(次数),
        ctypes.c_int32(类型),
        ctypes.c_char_p(module_name_bytes) if module_name_bytes else None
    )

    return ret

# INT64 __stdcall HCVMDMA_FindCodeX64Ex(int windowsIndex, int targetPid, INT64 beginAddr, int addrSize, CHAR* code, int offset, int num, int type = 1)
def HDDMA_搜索特征码x64Ex(窗口序号, 虚拟机中的目标进程PID, 开始地址, 地址大小, 特征码字符串, 偏移, 次数, 类型=1):
    """
    特征码搜索 (X64) 扩展版本

    :param 窗口序号: 窗口序号
    :param 虚拟机中的目标进程PID: 虚拟机中的目标进程PID
    :param 开始地址: 搜索的开始地址
    :param 地址大小: 搜索范围大小（单位：字节）
    :param 特征码字符串: 特征码字符串地址(支持??)
    :param 偏移: 正负偏移
    :param 次数: 找到的第几次，从1开始
    :param 类型: 获取的是(基地址/CALL地址/直接地址/地址中的值) 1为地址，2为基地址，3为CALL，4为地址中的值
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_FindCodeX64Ex = hd.HCVMDMA_FindCodeX64Ex
    HCVMDMA_FindCodeX64Ex.restype = ctypes.c_int64
    HCVMDMA_FindCodeX64Ex.argtypes = [
        ctypes.c_int32,  # 窗口序号
        ctypes.c_int32,  # 虚拟机中的目标进程PID
        ctypes.c_int64,  # 开始地址
        ctypes.c_int32,  # 地址大小
        ctypes.c_char_p,  # 特征码字符串
        ctypes.c_int32,  # 偏移
        ctypes.c_int32,  # 次数
        ctypes.c_int32  # 类型
    ]

    # 将特征码字符串转换为字节串
    code_bytes = auto_encode(特征码字符串)

    # 调用C函数
    ret = HCVMDMA_FindCodeX64Ex(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(虚拟机中的目标进程PID),
        ctypes.c_int64(开始地址),
        ctypes.c_int32(地址大小),
        code_bytes,
        ctypes.c_int32(偏移),
        ctypes.c_int32(次数),
        ctypes.c_int32(类型)
    )

    return ret

# INT64 __stdcall HCVMDMA_Close(int windowsIndex)
def HDDMA_断开虚拟机(窗口序号):
    """
    卸载并关闭连接虚拟机

    :param 窗口序号: 窗口序号
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_Close = hd.HCVMDMA_Close
    HCVMDMA_Close.restype = ctypes.c_int64
    HCVMDMA_Close.argtypes = [ctypes.c_int32]  # 窗口序号

    # 调用C函数
    ret = HCVMDMA_Close(ctypes.c_int32(窗口序号))

    return ret

# INT64 __stdcall HCVMDMA_InitCapture(int windowsIndex)
def HDDMA_获取截图环境(窗口序号):
    """
    初始化截图环境

    :param 窗口序号: 窗口序号
    :return: 截图环境句柄 (成功: 返回句柄，失败: 返回错误代码)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_InitCapture = hd.HCVMDMA_InitCapture
    HCVMDMA_InitCapture.restype = ctypes.c_int64
    HCVMDMA_InitCapture.argtypes = [ctypes.c_int32]  # 窗口序号

    # 调用C函数
    ret = HCVMDMA_InitCapture(ctypes.c_int32(窗口序号))

    return ret

# INT64 __stdcall HCVMDMA_InitMouseKey(int windowsIndex)
def HDDMA_获取键鼠环境(窗口序号):
    """
    初始化键鼠环境

    :param 窗口序号: 窗口序号
    :return: 键鼠环境句柄 (成功: 返回句柄，失败: 返回错误代码)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_InitMouseKey = hd.HCVMDMA_InitMouseKey
    HCVMDMA_InitMouseKey.restype = ctypes.c_int64
    HCVMDMA_InitMouseKey.argtypes = [ctypes.c_int32]  # 窗口序号

    # 调用C函数
    ret = HCVMDMA_InitMouseKey(ctypes.c_int32(窗口序号))

    return ret

# INT64 __stdcall HCVMDMA_Capture(int windowsIndex, INT64 captureEnvAddr)
def HDDMA_截图一次(窗口序号, 截图环境句柄):
    """
    截图一次

    :param 窗口序号: 窗口序号
    :param 截图环境句柄: 截图环境句柄，由 `HCVMDMA_InitCapture` 获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_Capture = hd.HCVMDMA_Capture
    HCVMDMA_Capture.restype = ctypes.c_int64
    HCVMDMA_Capture.argtypes = [ctypes.c_int32, ctypes.c_int64]  # 窗口序号, 截图环境句柄

    # 调用C函数
    ret = HCVMDMA_Capture(ctypes.c_int32(窗口序号), ctypes.c_int64(截图环境句柄))

    return ret

# INT64 __stdcall HCVMDMA_OpenCapture(int windowsIndex, INT64 captureEnvAddr)
def HDDMA_开启截图(窗口序号, 截图环境句柄):
    """
    开启截图(内部会开启一条线程循环调用HCVMDMA_Capture)
    需要先初始化关联虚拟机HCVMDMA_Init

    :param windowsIndex: 窗口序号
    :param captureEnvAddr: 截图环境句柄，由HCVMDMA_InitCapture获取返回
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_OpenCapture = hd.HCVMDMA_OpenCapture
    HCVMDMA_OpenCapture.restype = ctypes.c_int64
    HCVMDMA_OpenCapture.argtypes = [ctypes.c_int32, ctypes.c_int64]  # 窗口序号, 截图环境句柄

    # 调用C函数
    ret = HCVMDMA_OpenCapture(ctypes.c_int32(窗口序号), ctypes.c_int64(截图环境句柄))

    return ret

# INT64 __stdcall HCVMDMA_CloseCapture(int windowsIndex)
def HDDMA_关闭截图(窗口序号):
    """
    关闭截图(内部会关闭之前开启的线程)
    和HCVMDMA_OpenCapture是一对接口
    需要先初始化关联虚拟机HCVMDMA_Init

    :param windowsIndex: 窗口序号
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_CloseCapture = hd.HCVMDMA_CloseCapture
    HCVMDMA_CloseCapture.restype = ctypes.c_int64
    HCVMDMA_CloseCapture.argtypes = [ctypes.c_int32]  # 窗口序号

    # 调用C函数
    ret = HCVMDMA_CloseCapture(ctypes.c_int32(窗口序号))

    return ret

# INT64 __stdcall HCVMDMA_MoveTo(int windowsIndex, int x, int y, INT64 mkbEnvAddr)
def HDDMA_鼠标移动(窗口序号, x, y, 键鼠环境):
    """
    鼠标移动 (不包含轨迹)
    需要先初始化关联虚拟机HCVMDMA_Init

    :param windowsIndex: 窗口序号
    :param x: 虚拟机屏幕坐标X
    :param y: 虚拟机屏幕坐标Y
    :param mkbEnvAddr: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_MoveTo = hd.HCVMDMA_MoveTo
    HCVMDMA_MoveTo.restype = ctypes.c_int64
    HCVMDMA_MoveTo.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int64]  # 窗口序号, x, y, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_MoveTo(ctypes.c_int32(窗口序号), ctypes.c_int32(x), ctypes.c_int32(y), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_LeftDown(int windowsIndex, INT64 mkbEnvAddr)
def HDDMA_左键按下(窗口序号, 键鼠环境):
    """
    鼠标左键按下
    需要先初始化关联虚拟机HCVMDMA_Init

    :param windowsIndex: 窗口序号
    :param mkbEnvAddr: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_LeftDown = hd.HCVMDMA_LeftDown
    HCVMDMA_LeftDown.restype = ctypes.c_int64
    HCVMDMA_LeftDown.argtypes = [ctypes.c_int32, ctypes.c_int64]  # 窗口序号, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_LeftDown(ctypes.c_int32(窗口序号), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_LeftUp(int windowsIndex, INT64 mkbEnvAddr)
def HDDMA_左键弹起(窗口序号, 键鼠环境):
    """
    鼠标左键弹起
    需要先初始化关联虚拟机HCVMDMA_Init

    :param windowsIndex: 窗口序号
    :param mkbEnvAddr: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_LeftUp = hd.HCVMDMA_LeftUp
    HCVMDMA_LeftUp.restype = ctypes.c_int64
    HCVMDMA_LeftUp.argtypes = [ctypes.c_int32, ctypes.c_int64]  # 窗口序号, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_LeftUp(ctypes.c_int32(窗口序号), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_LeftClick(int windowsIndex, int mis, INT64 mkbEnvAddr)
def HDDMA_左键点击(窗口序号, mis, 键鼠环境):
    """
    鼠标左键点击
    需要先初始化关联虚拟机HCVMDMA_Init

    :param windowsIndex: 窗口序号
    :param mis: 按下和弹起之前的间隔时间(毫秒)
    :param mkbEnvAddr: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_LeftClick = hd.HCVMDMA_LeftClick
    HCVMDMA_LeftClick.restype = ctypes.c_int64
    HCVMDMA_LeftClick.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int64]  # 窗口序号, mis, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_LeftClick(ctypes.c_int32(窗口序号), ctypes.c_int32(mis), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_LeftDoubleClick(int windowsIndex, int mis, INT64 mkbEnvAddr)
def HDDMA_左键双击(窗口序号: int, mis: int, 键鼠环境: int) -> int:
    """
    鼠标左键双击
    需要先初始化关联虚拟机HCVMDMA_Init

    :param 窗口序号: 窗口序号
    :param mis: 按下和弹起之前的间隔时间(毫秒)
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_LeftDoubleClick = hd.HCVMDMA_LeftDoubleClick
    HCVMDMA_LeftDoubleClick.restype = ctypes.c_int64
    HCVMDMA_LeftDoubleClick.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int64]  # 窗口序号, mis, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_LeftDoubleClick(ctypes.c_int32(窗口序号), ctypes.c_int32(mis), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_MiddleDown(int windowsIndex, INT64 mkbEnvAddr)
def HDDMA_中键按下(窗口序号: int, 键鼠环境: int) -> int:
    """
    鼠标中键按下
    需要先初始化关联虚拟机HCVMDMA_Init

    :param 窗口序号: 窗口序号
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_MiddleDown = hd.HCVMDMA_MiddleDown
    HCVMDMA_MiddleDown.restype = ctypes.c_int64
    HCVMDMA_MiddleDown.argtypes = [ctypes.c_int32, ctypes.c_int64]  # 窗口序号, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_MiddleDown(ctypes.c_int32(窗口序号), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_MiddleUp(int windowsIndex, INT64 mkbEnvAddr)
def HDDMA_中键弹起(窗口序号: int, 键鼠环境: int) -> int:
    """
    鼠标中键弹起
    需要先初始化关联虚拟机HCVMDMA_Init

    :param 窗口序号: 窗口序号
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_MiddleUp = hd.HCVMDMA_MiddleUp
    HCVMDMA_MiddleUp.restype = ctypes.c_int64
    HCVMDMA_MiddleUp.argtypes = [ctypes.c_int32, ctypes.c_int64]  # 窗口序号, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_MiddleUp(ctypes.c_int32(窗口序号), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_MiddleClick(int windowsIndex, int mis, INT64 mkbEnvAddr)
def HDDMA_中键点击(窗口序号: int, mis: int, 键鼠环境: int) -> int:
    """
    鼠标中键点击
    需要先初始化关联虚拟机HCVMDMA_Init

    :param 窗口序号: 窗口序号
    :param mis: 按下和弹起之前的间隔时间(毫秒)
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_MiddleClick = hd.HCVMDMA_MiddleClick
    HCVMDMA_MiddleClick.restype = ctypes.c_int64
    HCVMDMA_MiddleClick.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int64]  # 窗口序号, mis, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_MiddleClick(ctypes.c_int32(窗口序号), ctypes.c_int32(mis), ctypes.c_int64(键鼠环境))

    return ret

# INT64 __stdcall HCVMDMA_MiddleDoubleClick(int windowsIndex, int mis, INT64 mkbEnvAddr)
def HDDMA_中键双击(窗口序号: int, mis: int, 键鼠环境: int) -> int:
    """
    鼠标中键双击
    需要先初始化关联虚拟机HCVMDMA_Init

    :param 窗口序号: 窗口序号
    :param mis: 按下和弹起之前的间隔时间(毫秒)
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码(查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_MiddleDoubleClick = hd.HCVMDMA_MiddleDoubleClick
    HCVMDMA_MiddleDoubleClick.restype = ctypes.c_int64
    HCVMDMA_MiddleDoubleClick.argtypes = [ctypes.c_int32, ctypes.c_int32, ctypes.c_int64]  # 窗口序号, mis, 键鼠环境

    # 调用C函数
    ret = HCVMDMA_MiddleDoubleClick(ctypes.c_int32(窗口序号), ctypes.c_int32(mis), ctypes.c_int64(键鼠环境))

    return ret

# INT64  __stdcall  HCVMDMA_RightDown(int windowsIndex, INT64 mkbEnvAddr);
def HDDMA_右键按下(窗口序号, mkbEnvAddr):
    """
    鼠标右键按下 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param mkbEnvAddr: 键鼠环境，由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_RightDown = hd.HCVMDMA_RightDown
    HCVMDMA_RightDown.restype = ctypes.c_int64
    HCVMDMA_RightDown.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_RightDown(
        ctypes.c_int32(窗口序号),
        ctypes.c_int64(mkbEnvAddr)
    )
    return ret

# INT64  __stdcall  HCVMDMA_RightUp(int windowsIndex, INT64 mkbEnvAddr);
def HDDMA_右键弹起(窗口序号, mkbEnvAddr):
    """
    鼠标右键弹起 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param mkbEnvAddr: 键鼠环境，由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_RightUp = hd.HCVMDMA_RightUp
    HCVMDMA_RightUp.restype = ctypes.c_int64
    HCVMDMA_RightUp.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_RightUp(
        ctypes.c_int32(窗口序号),
        ctypes.c_int64(mkbEnvAddr)
    )
    return ret

# INT64  __stdcall  HCVMDMA_RightClick(int windowsIndex, int mis, INT64 mkbEnvAddr);
def HDDMA_右键点击(窗口序号, 间隔时间, mkbEnvAddr):
    """
    鼠标右键点击 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 间隔时间: 按下和弹起之前的间隔时间 (毫秒)
    :param mkbEnvAddr: 键鼠环境，由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_RightClick = hd.HCVMDMA_RightClick
    HCVMDMA_RightClick.restype = ctypes.c_int64
    HCVMDMA_RightClick.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 间隔时间 (毫秒)
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_RightClick(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(间隔时间),
        ctypes.c_int64(mkbEnvAddr)
    )
    return ret

# INT64  __stdcall  HCVMDMA_RightDoubleClick(int windowsIndex, int mis, INT64 mkbEnvAddr);
def HDDMA_右键双击(窗口序号, 间隔时间, mkbEnvAddr):
    """
    鼠标右键双击 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 间隔时间: 按下和弹起之前的间隔时间 (毫秒)
    :param mkbEnvAddr: 键鼠环境，由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_RightDoubleClick = hd.HCVMDMA_RightDoubleClick
    HCVMDMA_RightDoubleClick.restype = ctypes.c_int64
    HCVMDMA_RightDoubleClick.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 间隔时间 (毫秒)
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_RightDoubleClick(
        ctypes.c_int32(窗口序号),
        ctypes.c_int32(间隔时间),
        ctypes.c_int64(mkbEnvAddr)
    )
    return ret

# INT64  __stdcall  HCVMDMA_WheelUp(int windowsIndex, INT64 mkbEnvAddr);
def HDDMA_滚轮滚上(窗口序号: int, 键鼠环境: int) -> int:
    """
    鼠标滚轮滚上 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_WheelUp = hd.HCVMDMA_WheelUp
    HCVMDMA_WheelUp.restype = ctypes.c_int64
    HCVMDMA_WheelUp.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_WheelUp(ctypes.c_int32(窗口序号), ctypes.c_int64(键鼠环境))
    return ret

# INT64  __stdcall  HCVMDMA_WheelDown(int windowsIndex, INT64 mkbEnvAddr);
def HDDMA_滚轮滚下(窗口序号: int, 键鼠环境: int) -> int:
    """
    鼠标滚轮滚下 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_WheelDown = hd.HCVMDMA_WheelDown
    HCVMDMA_WheelDown.restype = ctypes.c_int64
    HCVMDMA_WheelDown.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_WheelDown(ctypes.c_int32(窗口序号), ctypes.c_int64(键鼠环境))
    return ret

# INT64  __stdcall  HCVMDMA_KeyDown(int windowsIndex, int keyCode, INT64 mkbEnvAddr);
def HDDMA_键盘按下(窗口序号: int, 虚拟键码值: int, 键鼠环境: int) -> int:
    """
    键盘按下 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟键码值: 虚拟键码值
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_KeyDown = hd.HCVMDMA_KeyDown
    HCVMDMA_KeyDown.restype = ctypes.c_int64
    HCVMDMA_KeyDown.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 虚拟键码值
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_KeyDown(ctypes.c_int32(窗口序号), ctypes.c_int32(虚拟键码值), ctypes.c_int64(键鼠环境))
    return ret

# INT64  __stdcall  HCVMDMA_KeyUp(int windowsIndex, int keyCode, INT64 mkbEnvAddr);
def HDDMA_键盘按上(窗口序号: int, 虚拟键码值: int, 键鼠环境: int) -> int:
    """
    键盘按上 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟键码值: 虚拟键码值
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_KeyUp = hd.HCVMDMA_KeyUp
    HCVMDMA_KeyUp.restype = ctypes.c_int64
    HCVMDMA_KeyUp.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 虚拟键码值
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_KeyUp(ctypes.c_int32(窗口序号), ctypes.c_int32(虚拟键码值), ctypes.c_int64(键鼠环境))
    return ret

# INT64  __stdcall  HCVMDMA_KeyClick(int windowsIndex, int keyCode, int mis, INT64 mkbEnvAddr);
def HDDMA_键盘敲击(窗口序号: int, 虚拟键码值: int, mis: int, 键鼠环境: int) -> int:
    """
    键盘敲击 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param 虚拟键码值: 虚拟键码值
    :param mis: 按下和弹起之前的间隔时间(毫秒)
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_KeyClick = hd.HCVMDMA_KeyClick
    HCVMDMA_KeyClick.restype = ctypes.c_int64
    HCVMDMA_KeyClick.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_int32,   # 虚拟键码值
        ctypes.c_int32,   # 按下和弹起之前的间隔时间
        ctypes.c_int64    # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_KeyClick(ctypes.c_int32(窗口序号), ctypes.c_int32(虚拟键码值), ctypes.c_int32(mis), ctypes.c_int64(键鼠环境))
    return ret

# INT64  __stdcall  HCVMDMA_SendUnicode(int windowsIndex, wchar_t* strUnicode, INT64 mkbEnvAddr);
def HDDMA_发送文本Unicode(窗口序号: int, strUnicode: str, 键鼠环境: int) -> int:
    """
    发送unicode字符串 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param strUnicode: unicode编码字符串 (最大100长度, 一个字2个字节)
    :param 键鼠环境: 键鼠环境, 由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_SendUnicode = hd.HCVMDMA_SendUnicode
    HCVMDMA_SendUnicode.restype = ctypes.c_int64
    HCVMDMA_SendUnicode.argtypes = [
        ctypes.c_int32,       # 窗口序号
        ctypes.c_wchar_p,     # unicode编码字符串
        ctypes.c_int64        # 键鼠环境
    ]

    # 调用C函数
    ret = HCVMDMA_SendUnicode(ctypes.c_int32(窗口序号), strUnicode, ctypes.c_int64(键鼠环境))
    return ret

# INT64  __stdcall  HCVMDMA_SendAscii(int windowsIndex, char* strAscii, INT64 mkbEnvAddr);
def HDDMA_发送文本Ascii(窗口序号: int, strAscii: str, 键鼠环境: int) -> int:
    """
    发送ascii字符串 需要先初始化关联虚拟机HCVMDMA_Init
    :param 窗口序号: 窗口序号
    :param strAscii: ascii编码字符串 (最大200字符，每个字符1字节，共200字节)，例如数字、英文、符号等
    :param 键鼠环境: 键鼠环境，由HCVMDMA_InitMouseKey获取
    :return: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    # 定义函数指针类型
    HCVMDMA_SendAscii = hd.HCVMDMA_SendAscii
    HCVMDMA_SendAscii.restype = ctypes.c_int64
    HCVMDMA_SendAscii.argtypes = [
        ctypes.c_int32,   # 窗口序号
        ctypes.c_char_p,  # ascii字符串
        ctypes.c_int64    # 键鼠环境
    ]

    # 将字符串转换为字节串
    ascii_bytes = auto_encode(strAscii)

    # 调用C函数
    ret = HCVMDMA_SendAscii(
        ctypes.c_int32(窗口序号),
        ctypes.c_char_p(ascii_bytes),
        ctypes.c_int64(键鼠环境)
    )
    return ret

def HDDMA_获取PID列表(
    window_index: int
) -> int:
    """
    获取PID列表
    Args:
        window_index (int): 窗口序号
    Returns:
        int: PID数量（大于0为数量，小于等于0为错误），具体进程信息需用HCEnv_GetRetJson获取
        返回JSON:具体进程PID从最近json获取字符串 pid1,name1|pid2,name2|pid3,name3|  "|"分隔符 每个进程信息
    """
    hd = Config.get_hd()
    HCVMDMA_GetPidList = hd.HCVMDMA_GetPidList
    HCVMDMA_GetPidList.restype = ctypes.c_int64
    HCVMDMA_GetPidList.argtypes = [ctypes.c_int32]
    ret = HCVMDMA_GetPidList(ctypes.c_int32(window_index))
    return ret

def HDDMA_PID是否存在(
    window_index: int,
    pid: int
) -> int:
    """
    接口说明:
        对应虚拟机中的进程PID是否存在

    函数原型:
        __int64  __stdcall  HCVMDMA_IsExistPid(__int32 windowsIndex,__int32 pid);

    参数说明:
        window_index: 窗口序号
        pid: 进程PID

    返回值:
        int: 0 或 1（查看HD返回值表）

    备注:
        需要先初始化关联虚拟机HCVMDMA_Init
    """
    hd = Config.get_hd()
    HCVMDMA_IsExistPid = hd.HCVMDMA_IsExistPid
    HCVMDMA_IsExistPid.restype = ctypes.c_int64
    HCVMDMA_IsExistPid.argtypes = [ctypes.c_int32, ctypes.c_int32]
    ret = HCVMDMA_IsExistPid(ctypes.c_int32(window_index), ctypes.c_int32(pid))
    return ret


from .config import Config, ctypes
from .config import auto_encode

# HD多线程流程回调
# __int64(__stdcall *FUNTYPE )(__int32 windowsIndex);
HD_ProcessFunType_stdcall = ctypes.WINFUNCTYPE(ctypes.c_int64, ctypes.c_int32)

# HD多线程操作回调
# __int64(__stdcall *FUNTYPE )(__int32 index);
HD_OperationFunType_stdcall = ctypes.WINFUNCTYPE(ctypes.c_int64, ctypes.c_int32)

# HD多线程消息回调X64
# typedef __int64(__stdcall *MSGFUNTYPE )(WPARAM wparam,LPARAM lparam);
HD_MsgFunType_stdcall = ctypes.WINFUNCTYPE(ctypes.c_int64, ctypes.c_void_p, ctypes.c_void_p)

# HD多线程UI回调
# typedef __int64(__stdcall *UIFUNTYPE)(__int32 windowsIndex, char *stepText, BOOL end1, BOOL pause1, __int32 threadState1, BOOL m_end2, BOOL m_pause2, __int32 threadState2);
HD_UiFunType_stdcall = ctypes.WINFUNCTYPE(ctypes.c_int64, ctypes.c_int32, ctypes.c_char_p, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32)

# HD多线程UIEx回调
# typedef __int64(__stdcall *UIFUNTYPE)(__int32 windowsIndex, char *stepText, void* lparam,  BOOL end1, BOOL pause1, __int32 threadState1, BOOL m_end2, BOOL m_pause2, __int32 threadState2);
HD_UiExFunType_stdcall = ctypes.WINFUNCTYPE(ctypes.c_int64, ctypes.c_int32, ctypes.c_char_p, ctypes.c_void_p, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32, ctypes.c_int32)

UI回调Fun = None
登录回调Fun = None
第一执行回调Fun = None
第二执行回调Fun = None
结束回调Fun = None
重启前回调Fun = None

# INT64 __stdcall HCMT_InitProcess(INT64 hwnd, void* updateUICallBack, void* loginCallBack, void* firstCallBack, void* secondCallBack, void* endCallBack, void* restartPreCallBack);
def HD多线程_初始化流程回调(窗口主句柄: int, UI回调: HD_UiFunType_stdcall, 登录回调: HD_ProcessFunType_stdcall, 第一执行回调: HD_ProcessFunType_stdcall, 第二执行回调: HD_ProcessFunType_stdcall, 结束回调: HD_ProcessFunType_stdcall, 重启前回调: HD_ProcessFunType_stdcall) -> int:
    """
    初始化多线程设置相关流程回调
    
    Args:
        窗口主句柄: 中控UI的窗口主句柄，内部会HOOK窗口过程
        UI回调: UI回调函数指针
        登录回调: 登录回调函数指针
        第一执行回调: 第一执行回调函数指针，注意：内部应该写成循环并配合HCMT_Sleep()和HCMT_IsRunning()，不要主动返回执行回调(return 1;)，应该调用接口内部结束或者其他操作
        第二执行回调: 第二执行回调函数指针，注意：内部应该写成循环并配合HCMT_Sleep()和HCMT_IsRunning()，不要主动返回执行回调(return 1;)，应该调用接口内部结束或者其他操作
        结束回调: 结束回调函数指针
        重启前回调: 重启前回调函数指针
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_InitProcess = hd.HCMT_InitProcess
    HCMT_InitProcess.restype = ctypes.c_int64
    HCMT_InitProcess.argtypes = [
        ctypes.c_int64,
        HD_UiFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
    ]
    
    global UI回调Fun, 登录回调Fun, 第一执行回调Fun, 第二执行回调Fun, 结束回调Fun, 重启前回调Fun
    UI回调Fun = HD_UiFunType_stdcall(UI回调)
    登录回调Fun = HD_ProcessFunType_stdcall(登录回调)
    第一执行回调Fun = HD_ProcessFunType_stdcall(第一执行回调)
    第二执行回调Fun = HD_ProcessFunType_stdcall(第二执行回调)
    结束回调Fun = HD_ProcessFunType_stdcall(结束回调)
    重启前回调Fun = HD_ProcessFunType_stdcall(重启前回调)
    
    ret = HCMT_InitProcess(
        ctypes.c_int64(窗口主句柄),
        UI回调Fun,
        登录回调Fun,
        第一执行回调Fun,
        第二执行回调Fun,
        结束回调Fun,
        重启前回调Fun
    )
    return ret

# INT64 __stdcall HCMT_InitProcessEx(INT64 hwnd, void* updateUICallBackEx, void* loginCallBack, void* firstCallBack, void* secondCallBack, void* endCallBack, void* restartPreCallBack, void* lparam);
def HD多线程_初始化流程回调Ex(窗口主句柄: int, UI回调: HD_UiExFunType_stdcall, 登录回调: HD_ProcessFunType_stdcall, 第一执行回调: HD_ProcessFunType_stdcall, 第二执行回调: HD_ProcessFunType_stdcall, 结束回调: HD_ProcessFunType_stdcall, 重启前回调: HD_ProcessFunType_stdcall, 额外参数: ctypes.c_void_p) -> int:
    """
    初始化多线程扩展版本设置相关流程回调
    
    Args:
        窗口主句柄: 中控UI的窗口主句柄，内部会HOOK窗口过程
        UI回调: UI回调函数指针
        登录回调: 登录回调函数指针
        第一执行回调: 第一执行回调函数指针，注意：内部应该写成循环并配合HCMT_Sleep()和HCMT_IsRunning()，不要主动返回执行回调(return 1;)，应该调用接口内部结束或者其他操作
        第二执行回调: 第二执行回调函数指针，注意：内部应该写成循环并配合HCMT_Sleep()和HCMT_IsRunning()，不要主动返回执行回调(return 1;)，应该调用接口内部结束或者其他操作
        结束回调: 结束回调函数指针
        重启前回调: 重启前回调函数指针
        额外参数: 绑定一个全局参数，一般我们指定为一个UI对象地址
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_InitProcessEx = hd.HCMT_InitProcessEx
    HCMT_InitProcessEx.restype = ctypes.c_int64
    HCMT_InitProcessEx.argtypes = [
        ctypes.c_int64,
        HD_UiExFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        HD_ProcessFunType_stdcall,
        ctypes.c_void_p
    ]
    
    global UI回调Fun, 登录回调Fun, 第一执行回调Fun, 第二执行回调Fun, 结束回调Fun, 重启前回调Fun
    UI回调Fun = HD_UiExFunType_stdcall(UI回调)
    登录回调Fun = HD_ProcessFunType_stdcall(登录回调)
    第一执行回调Fun = HD_ProcessFunType_stdcall(第一执行回调)
    第二执行回调Fun = HD_ProcessFunType_stdcall(第二执行回调)
    结束回调Fun = HD_ProcessFunType_stdcall(结束回调)
    重启前回调Fun = HD_ProcessFunType_stdcall(重启前回调)
    
    ret = HCMT_InitProcessEx(
        ctypes.c_int64(窗口主句柄),
        UI回调Fun,
        登录回调Fun,
        第一执行回调Fun,
        第二执行回调Fun,
        结束回调Fun,
        重启前回调Fun,
        额外参数
    )
    return ret

结束绑定回调Fun = None
暂停绑定回调Fun = None
恢复绑定回调Fun = None

# INT64 __stdcall HCMT_InitOperate(void* endBindCallBack, void* pauseBindCallBack, void* recoverBindCallBack);
def HD多线程_初始化操作回调(结束绑定回调: HD_OperationFunType_stdcall, 暂停绑定回调: HD_OperationFunType_stdcall, 恢复绑定回调: HD_OperationFunType_stdcall) -> int:
    """
    初始化多线程结束/暂停/恢复状态的操作回调
    
    Args:
        结束绑定回调: 结束绑定回调函数指针
        暂停绑定回调: 暂停绑定回调函数指针
        恢复绑定回调: 恢复绑定回调函数指针
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_InitOperate = hd.HCMT_InitOperate
    HCMT_InitOperate.restype = ctypes.c_int64
    HCMT_InitOperate.argtypes = [
        HD_OperationFunType_stdcall,
        HD_OperationFunType_stdcall,
        HD_OperationFunType_stdcall
    ]
    
    global 结束绑定回调Fun, 暂停绑定回调Fun, 恢复绑定回调Fun
    结束绑定回调Fun = HD_OperationFunType_stdcall(结束绑定回调)
    暂停绑定回调Fun = HD_OperationFunType_stdcall(暂停绑定回调)
    恢复绑定回调Fun = HD_OperationFunType_stdcall(恢复绑定回调)
    
    ret = HCMT_InitOperate(
        结束绑定回调Fun,
        暂停绑定回调Fun,
        恢复绑定回调Fun
    )
    return ret

# INT64 __stdcall HCMT_RegisterMessage(int msg, void* msgCallBack);
def HD多线程_注册消息回调(自定义消息整数值: int, 消息回调: HD_MsgFunType_stdcall) -> int:
    """
    注册窗口消息
    
    Args:
        自定义消息整数值: 自定义消息整数值，规定值大于等于41124
        消息回调: 消息回调函数指针
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_RegisterMessage = hd.HCMT_RegisterMessage
    HCMT_RegisterMessage.restype = ctypes.c_int64
    HCMT_RegisterMessage.argtypes = [
        ctypes.c_int32,
        ctypes.c_void_p
    ]
    
    消息回调Fun = HD_MsgFunType_stdcall(消息回调)
    消息回调Fun_ptr = ctypes.cast(消息回调Fun, ctypes.c_void_p).value if 消息回调Fun else ctypes.c_void_p(0)
    
    ret = HCMT_RegisterMessage(
        ctypes.c_int32(自定义消息整数值),
        消息回调Fun_ptr
    )
    return ret

# INT64 __stdcall HCMT_MsgSend(int msg, void* wparam, void* lparam);
def HD多线程_同步发送消息(自定义消息整数值: int, 自定义参数1: int, 自定义参数2: int) -> int:
    """
    发送消息同步
    
    Args:
        自定义消息整数值: 消息整数值
        自定义参数1: 自定义参数1
        自定义参数2: 自定义参数2
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgSend = hd.HCMT_MsgSend
    HCMT_MsgSend.restype = ctypes.c_int64
    HCMT_MsgSend.argtypes = [
        ctypes.c_int32,
        ctypes.c_void_p,
        ctypes.c_void_p
    ]
    
    ret = HCMT_MsgSend(
        ctypes.c_int32(自定义消息整数值),
        ctypes.cast(自定义参数1, ctypes.c_void_p),
        ctypes.cast(自定义参数2, ctypes.c_void_p)
    )
    return ret

# INT64 __stdcall HCMT_MsgPost(int msg, void* wparam, void* lparam);
def HD多线程_异步发送消息(自定义消息整数值: int, 自定义参数1: int, 自定义参数2: int) -> int:
    """
    发送消息异步
    
    Args:
        自定义消息整数值: 消息整数值
        自定义参数1: 自定义参数1
        自定义参数2: 自定义参数2
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgPost = hd.HCMT_MsgPost
    HCMT_MsgPost.restype = ctypes.c_int64
    HCMT_MsgPost.argtypes = [
        ctypes.c_int32,
        ctypes.c_void_p,
        ctypes.c_void_p
    ]
    
    ret = HCMT_MsgPost(
        ctypes.c_int32(自定义消息整数值),
        ctypes.cast(自定义参数1, ctypes.c_void_p),
        ctypes.cast(自定义参数2, ctypes.c_void_p)
    )
    return ret

# INT64 __stdcall HCMT_MsgStart(int windowsIndex, BOOL bAsyn);
def HD多线程_开启窗口Msg(窗口序号: int, 是否异步发送: bool = False) -> int:
    """
    通过消息开启窗口操作
    
    Args:
        窗口序号: 窗口序号
        是否异步发送: 是否异步发送
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgStart = hd.HCMT_MsgStart
    HCMT_MsgStart.restype = ctypes.c_int64
    HCMT_MsgStart.argtypes = [
        ctypes.c_int32,
        ctypes.c_bool
    ]
    
    ret = HCMT_MsgStart(
        ctypes.c_int32(窗口序号),
        ctypes.c_bool(是否异步发送)
    )
    return ret

# INT64 __stdcall HCMT_MsgStop(int windowsIndex, BOOL bAsyn);
def HD多线程_停止窗口Msg(窗口序号: int, 是否异步发送: bool = False) -> int:
    """
    通过消息停止窗口操作
    
    Args:
        窗口序号: 窗口序号
        是否异步发送: 是否异步发送
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgStop = hd.HCMT_MsgStop
    HCMT_MsgStop.restype = ctypes.c_int64
    HCMT_MsgStop.argtypes = [
        ctypes.c_int32,
        ctypes.c_bool
    ]
    
    ret = HCMT_MsgStop(
        ctypes.c_int32(窗口序号),
        ctypes.c_bool(是否异步发送)
    )
    return ret

# INT64 __stdcall HCMT_MsgReStart(int windowsIndex, BOOL bAsyn);
def HD多线程_重启窗口Msg(窗口序号: int, 是否异步发送: bool = False) -> int:
    """
    通过消息重启窗口操作
    
    Args:
        窗口序号: 窗口序号
        是否异步发送: 是否异步发送
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgReStart = hd.HCMT_MsgReStart
    HCMT_MsgReStart.restype = ctypes.c_int64
    HCMT_MsgReStart.argtypes = [
        ctypes.c_int32,
        ctypes.c_bool
    ]
    
    ret = HCMT_MsgReStart(
        ctypes.c_int32(窗口序号),
        ctypes.c_bool(是否异步发送)
    )
    return ret

# INT64 __stdcall HCMT_MsgReStartEx(int windowsIndex, BOOL bUnload, BOOL bAsyn);
def HD多线程_重启窗口MsgEx(窗口序号: int, 是否卸载环境: bool = True, 是否重连: bool = True,是否异步发送: bool = False) -> int:
    """
    通过消息重启窗口操作扩展版
    
    Args:
        窗口序号: 窗口序号
        是否卸载环境: 是否卸载环境
        是否重连: 是否重连
        是否异步发送: 是否异步发送
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgReStartEx = hd.HCMT_MsgReStartEx
    HCMT_MsgReStartEx.restype = ctypes.c_int64
    HCMT_MsgReStartEx.argtypes = [
        ctypes.c_int32,
        ctypes.c_bool,
        ctypes.c_bool,
        ctypes.c_bool
    ]
    
    ret = HCMT_MsgReStartEx(
        ctypes.c_int32(窗口序号),
        ctypes.c_bool(是否卸载环境),
        ctypes.c_bool(是否重连),
        ctypes.c_bool(是否异步发送)
    )
    return ret


# INT64 __stdcall HCMT_MsgUpdateUI(int windowsIndex, BOOL bAsyn);
def HD多线程_更新UI(窗口序号: int, 是否异步发送: bool = False) -> int:
    """
    通过消息触发UI回调，并传递线程状态信息和日志操作信息给UI回调
    
    Args:
        窗口序号: 窗口序号
        是否异步发送: 是否异步发送
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgUpdateUI = hd.HCMT_MsgUpdateUI
    HCMT_MsgUpdateUI.restype = ctypes.c_int64
    HCMT_MsgUpdateUI.argtypes = [
        ctypes.c_int32,
        ctypes.c_bool
    ]
    
    ret = HCMT_MsgUpdateUI(
        ctypes.c_int32(窗口序号),
        ctypes.c_bool(是否异步发送)
    )
    return ret

# INT64 __stdcall HCMT_MsgStepText(int windowsIndex, char* text, BOOL bAsyn);
def HD多线程_发送文本(窗口序号: int, 文本: str, 是否异步发送: bool = False) -> int:
    """
    通过消息触发UI回调，并传递线程状态信息和日志操作信息给UI回调
    
    Args:
        窗口序号: 窗口序号
        文本: 要发送的文本字符串
        是否异步发送: 是否异步发送
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_MsgStepText = hd.HCMT_MsgStepText
    HCMT_MsgStepText.restype = ctypes.c_int64
    HCMT_MsgStepText.argtypes = [
        ctypes.c_int32,
        ctypes.c_char_p,
        ctypes.c_bool
    ]
    
    ret = HCMT_MsgStepText(
        ctypes.c_int32(窗口序号),
        auto_encode(文本),
        ctypes.c_bool(是否异步发送)
    )
    return ret

# INT64 __stdcall HCMT_SetAllPause();
def HD多线程_所有窗口暂停() -> int:
    """
    设置所有窗口暂停
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetAllPause = hd.HCMT_SetAllPause
    HCMT_SetAllPause.restype = ctypes.c_int64
    HCMT_SetAllPause.argtypes = []
    
    ret = HCMT_SetAllPause()
    return ret

# INT64 __stdcall HCMT_GetState(int index);
def HD多线程_获取线程状态值(主副序号: int) -> int:
    """
    获取主副序号对应的线程的状态值
    
    Args:
        主副序号: 主副序号
    
    Returns:
        int: 状态值
    """
    hd = Config.get_hd()
    
    HCMT_GetState = hd.HCMT_GetState
    HCMT_GetState.restype = ctypes.c_int64
    HCMT_GetState.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_GetState(ctypes.c_int32(主副序号))
    return ret

# char* __stdcall HCMT_GetStateString(int threadState);
def HD多线程_获取线程状态说明(状态值: int) -> str:
    """
    获取状态整数值对应的字符串
    
    Args:
        状态值: 是HD多线程_获取线程状态值返回的值
    
    Returns:
        str: 状态说明字符串
    """
    hd = Config.get_hd()
    
    HCMT_GetStateString = hd.HCMT_GetStateString
    HCMT_GetStateString.restype = ctypes.c_char_p
    HCMT_GetStateString.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_GetStateString(ctypes.c_int32(状态值))
    # C接口返回的char*需要解码为Python字符串
    if ret:
        return ret.decode('utf-8', errors='ignore')
    else:
        return ""

# INT64 __stdcall HCMT_Start(int windowsIndex);
def HD多线程_开启窗口(窗口序号: int) -> int:
    """
    直接开启窗口操作
    
    Args:
        窗口序号: 窗口序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_Start = hd.HCMT_Start
    HCMT_Start.restype = ctypes.c_int64
    HCMT_Start.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_Start(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCMT_SetAllRecover();
def HD多线程_所有窗口恢复() -> int:
    """
    设置所有窗口恢复
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetAllRecover = hd.HCMT_SetAllRecover
    HCMT_SetAllRecover.restype = ctypes.c_int64
    HCMT_SetAllRecover.argtypes = []
    
    ret = HCMT_SetAllRecover()
    return ret

# INT64 __stdcall HCMT_SetAllStop();
def HD多线程_所有窗口停止() -> int:
    """
    设置所有窗口停止
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetAllStop = hd.HCMT_SetAllStop
    HCMT_SetAllStop.restype = ctypes.c_int64
    HCMT_SetAllStop.argtypes = []
    
    ret = HCMT_SetAllStop()
    return ret

# INT64 __stdcall HCMT_SetPause(int windowsIndex);
def HD多线程_暂停窗口(窗口序号: int) -> int:
    """
    设置窗口暂停
    
    Args:
        窗口序号: 窗口序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetPause = hd.HCMT_SetPause
    HCMT_SetPause.restype = ctypes.c_int64
    HCMT_SetPause.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_SetPause(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCMT_SetPauseEx(int index);
def HD多线程_暂停线程(线程序号: int) -> int:
    """
    设置主副序号对应的线程暂停
    
    Args:
        线程序号: 线程序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetPauseEx = hd.HCMT_SetPauseEx
    HCMT_SetPauseEx.restype = ctypes.c_int64
    HCMT_SetPauseEx.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_SetPauseEx(ctypes.c_int32(线程序号))
    return ret

# INT64 __stdcall HCMT_SetRecover(int windowsIndex);
def HD多线程_恢复窗口(窗口序号: int) -> int:
    """
    设置窗口恢复
    
    Args:
        窗口序号: 窗口序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetRecover = hd.HCMT_SetRecover
    HCMT_SetRecover.restype = ctypes.c_int64
    HCMT_SetRecover.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_SetRecover(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCMT_SetRecoverEx(int index);
def HD多线程_恢复线程(线程序号: int) -> int:
    """
    设置主副序号对应的线程恢复
    
    Args:
        线程序号: 线程序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetRecoverEx = hd.HCMT_SetRecoverEx
    HCMT_SetRecoverEx.restype = ctypes.c_int64
    HCMT_SetRecoverEx.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_SetRecoverEx(ctypes.c_int32(线程序号))
    return ret

# INT64 __stdcall HCMT_SetStop(int windowsIndex);
def HD多线程_停止窗口(窗口序号: int) -> int:
    """
    设置窗口停止
    
    Args:
        窗口序号: 窗口序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_SetStop = hd.HCMT_SetStop
    HCMT_SetStop.restype = ctypes.c_int64
    HCMT_SetStop.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_SetStop(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCMT_IsRunning();
def HD多线程_是否运行中() -> int:
    """
    在第一第二回调里面调用，检查当前线程是否结束
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_IsRunning = hd.HCMT_IsRunning
    HCMT_IsRunning.restype = ctypes.c_int64
    HCMT_IsRunning.argtypes = []
    
    ret = HCMT_IsRunning()
    return ret

# INT64 __stdcall HCMT_IsCanStart(int windowsIndex);
def HD多线程_是否能开启(窗口序号: int) -> int:
    """
    在第一第二回调里面调用，检查当前线程是否结束
    
    Args:
        窗口序号: 窗口序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_IsCanStart = hd.HCMT_IsCanStart
    HCMT_IsCanStart.restype = ctypes.c_int64
    HCMT_IsCanStart.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_IsCanStart(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCMT_IsPause(int index);
def HD多线程_是否暂停(线程序号: int) -> int:
    """
    判断主副序号对应的线程是否是已被暂停状态
    
    Args:
        线程序号: 线程序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_IsPause = hd.HCMT_IsPause
    HCMT_IsPause.restype = ctypes.c_int64
    HCMT_IsPause.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_IsPause(ctypes.c_int32(线程序号))
    return ret

# INT64 __stdcall HCMT_IsStop(int index);
def HD多线程_是否停止(线程序号: int) -> int:
    """
    判断主副序号对应的线程是否是已被结束状态
    
    Args:
        线程序号: 线程序号
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCMT_IsStop = hd.HCMT_IsStop
    HCMT_IsStop.restype = ctypes.c_int64
    HCMT_IsStop.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_IsStop(ctypes.c_int32(线程序号))
    return ret

# INT64 __stdcall HCMT_Sleep(int mis);
def HD多线程_延迟(毫秒延迟: int) -> int:
    """
    延迟函数，自带暂停/结束/恢复检查
    
    Args:
        毫秒延迟: 毫秒延迟
    
    Returns:
        int: 返回值代码，1：正常，2：线程结束，3：线程被暂停过
    """
    hd = Config.get_hd()
    
    HCMT_Sleep = hd.HCMT_Sleep
    HCMT_Sleep.restype = ctypes.c_int64
    HCMT_Sleep.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCMT_Sleep(ctypes.c_int32(毫秒延迟))
    return ret





from .config import Config, ctypes
from .config import auto_encode

# char* __stdcall HCEnv_GetRetJson(__int32 窗口序号);
def HD通用_获取最近返回Json(窗口序号: int) -> bytes:
    """
    获取最近一次调用接口返回的json字符串
    
    Args:
        窗口序号: 窗口序号 可以包含0 表示中控
        
    Returns:
        bytes: JSON字符串 (字节格式)
    """
    hd = Config.get_hd()

    func = hd.HCEnv_GetRetJson
    func.restype = ctypes.c_char_p
    func.argtypes = [ctypes.c_int]

    return func(窗口序号)

# INT64 __stdcall HCEnv_GetRetValue(__int32 窗口序号);
def HD通用_获取最近返回值(窗口序号: int) -> int:
    """
    获取最近一次调用接口返回的值(用于检测是否有错)
    
    Args:
        窗口序号: 窗口序号 可以包含0 表示中控
        
    Returns:
        int: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    func = hd.HCEnv_GetRetValue
    func.restype = ctypes.c_int64
    func.argtypes = [ctypes.c_int]

    return func(窗口序号)

# char* __stdcall HCEnv_GetErrorStr(__int32 ret);
def HD通用_获取返回值说明(ret: int) -> bytes:
    """
    根据返回值获取错误说明
    
    Args:
        ret: 返回值代码
        
    Returns:
        bytes: 错误说明字符串 (字节格式)
    """
    hd = Config.get_hd()

    func = hd.HCEnv_GetErrorStr
    func.restype = ctypes.c_char_p
    func.argtypes = [ctypes.c_int]

    return func(ret)

# INT64 __stdcall HCEnv_GetLastError();
def HD通用_获取最近API返回值() -> int:
    """
    获取最近API调用的返回值
    
    Returns:
        int: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    func = hd.HCEnv_GetLastError
    func.restype = ctypes.c_int64
    func.argtypes = []

    return func()

# INT64 __stdcall HCInject_GetLastInfo(__int32 窗口序号);
def HD通用_获取当前插件状态值(窗口序号: int) -> int:
    """
    获取当前环境搭建流程中的操作信息
    
    Args:
        窗口序号: 窗口序号 (从1开始)
        
    Returns:
        int: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    func = hd.HCInject_GetLastInfo
    func.restype = ctypes.c_int64
    func.argtypes = [ctypes.c_int]

    return func(窗口序号)

# __int64 __stdcall HC_CALL(__int32 窗口序号, __int64 CALL地址, __int64 rcx = 0, __int64 rdx = 0, __int64 r8 = 0, __int64 r9 = 0, __int64 lparam5 = 0, __int64 lparam6 = 0, __int64 lparam7 = 0, __int64 lparam8 = 0, BOOL 是否主线程调用 = FALSE);
def HD通用_CALLX64(窗口序号: int, CALL地址: int, rcx: int = 0, rdx: int = 0, r8: int = 0, r9: int = 0, lparam5: int = 0, lparam6: int = 0, lparam7: int = 0, lparam8: int = 0, 是否主线程调用: bool = False) -> int:
    """
    通用CALL(适合X64),支持0-8个参数
    
    Args:
        窗口序号: 窗口序号 (从1开始)
        CALL地址: CALL地址
        rcx: CALL参数 (默认0)
        rdx: CALL参数 (默认0)
        r8: CALL参数 (默认0)
        r9: CALL参数 (默认0)
        lparam5: CALL参数 (默认0)
        lparam6: CALL参数 (默认0)
        lparam7: CALL参数 (默认0)
        lparam8: CALL参数 (默认0)
        是否主线程调用: 是否主线程调用这个CALL,需要挂接主线程 (默认False)
        
    Returns:
        int: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    func = hd.HC_CALL
    func.restype = ctypes.c_int64
    func.argtypes = [ctypes.c_int, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_int64, ctypes.c_bool]

    return func(窗口序号, CALL地址, rcx, rdx, r8, r9, lparam5, lparam6, lparam7, lparam8, 是否主线程调用)

# __int64 __stdcall HC_CALLX86(__int32 窗口序号, __int32 CALL地址, __int32 ecx = 0,__int32 lparam1 = 0, __int32 lparam2 = 0, __int32 lparam3 = 0, __int32 lparam4 = 0, __int32 lparam5 = 0, __int32 lparam6 = 0, __int32 lparam7 = 0, __int32 lparam8 = 0, __int32 lparamNum = 0, __int32 addEsp = -1, BOOL 是否主线程调用 = FALSE);
def HD通用_CALLX86(窗口序号: int, CALL地址: int, ecx: int = 0, lparam1: int = 0, lparam2: int = 0, lparam3: int = 0, lparam4: int = 0, lparam5: int = 0, lparam6: int = 0, lparam7: int = 0, lparam8: int = 0, lparamNum: int = 0, addEsp: int = -1, 是否主线程调用: bool = False) -> int:
    """
    通用CALL(适合X86),支持0到8个参数
    
    Args:
        窗口序号: 窗口序号 (从1开始)
        CALL地址: 4字节接口地址
        ecx: 对象参数 (默认0)
        lparam1: 第一个参数 (默认0)
        lparam2: 第二个参数 (默认0)
        lparam3: 第三个参数 (默认0)
        lparam4: 第四个参数 (默认0)
        lparam5: 第五个参数 (默认0)
        lparam6: 第六个参数 (默认0)
        lparam7: 第七个参数 (默认0)
        lparam8: 第八个参数 (默认0)
        lparamNum: 当前接口地址的传递的参数数量(支持0到8个参数) (默认0)
        addEsp: 如果调用的接口地址是内平栈就置为-1,如果是外平栈就设置为参数数量(lparamNum)*0x4 (默认-1)
        是否主线程调用: 是否主线程调用这个CALL,需要挂接主线程 (默认False)
        
    Returns:
        int: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    func = hd.HC_CALLX86
    func.restype = ctypes.c_int64
    func.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_bool]

    return func(窗口序号, CALL地址, ecx, lparam1, lparam2, lparam3, lparam4, lparam5, lparam6, lparam7, lparam8, lparamNum, addEsp, 是否主线程调用)

# __int64 __stdcall HC_HookMainThread(__int32 窗口序号, __int64 窗口句柄);
def HD通用_挂接主线程(窗口序号: int, 窗口句柄: int) -> int:
    """
    挂接主线程
    
    Args:
        窗口序号: 窗口序号 (从1开始)
        窗口句柄: 窗口句柄
        
    Returns:
        int: 返回值代码 (查看HD返回值表)
    """
    hd = Config.get_hd()

    func = hd.HC_HookMainThread
    func.restype = ctypes.c_int64
    func.argtypes = [ctypes.c_int, ctypes.c_int64]

    return func(窗口序号, 窗口句柄)


from .config import Config, ctypes
from .config import auto_encode

# INT64 __stdcall HCHDPP_Protect(int pid);
def HD盾_保护(进程PID: int) -> int:
    """
    一键保护进程（隐藏等）
    
    Args:
        进程PID: 需要保护的进程PID
    
    Returns:
        int: 返回值代码（查看HD返回值表）
    
    Note:
        需要调用HCHD_LoadDrv2传递参数3来安装保护盾HDPP驱动
    """
    hd = Config.get_hd()
    
    HCHDPP_Protect = hd.HCHDPP_Protect
    HCHDPP_Protect.restype = ctypes.c_int64
    HCHDPP_Protect.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCHDPP_Protect(ctypes.c_int32(进程PID))
    return ret

# INT64 __stdcall HCHDPW_OnProtect(INT64 hwnd);
def HD盾_开启窗口保护(窗口句柄: int) -> int:
    """
    打开窗口保护以及子窗口
    
    Args:
        窗口句柄: 目标窗口句柄，一般为父窗口句柄
    
    Returns:
        int: 返回值代码（查看HD返回值表）
    
    Note:
        需要调用HCHD_LoadDrv2传递参数5来安装保护盾HDPW驱动
        可以多次指定，内部会把指定的保护窗口句柄以及所属子窗口一起保护
    """
    hd = Config.get_hd()
    
    HCHDPW_OnProtect = hd.HCHDPW_OnProtect
    HCHDPW_OnProtect.restype = ctypes.c_int64
    HCHDPW_OnProtect.argtypes = [
        ctypes.c_int64
    ]
    
    ret = HCHDPW_OnProtect(ctypes.c_int64(窗口句柄))
    return ret

# INT64 __stdcall HCHDPW_OffProtect();
def HD盾_关闭窗口保护() -> int:
    """
    取消所有窗口保护（指定保护过的）
    
    Returns:
        int: 返回值代码（查看HD返回值表）
    
    Note:
        需要调用HCHD_LoadDrv2传递参数5来安装保护盾HDPW驱动
        可以取消调用过HCHDPW_OnProtect所有保护的窗口句柄
    """
    hd = Config.get_hd()
    
    HCHDPW_OffProtect = hd.HCHDPW_OffProtect
    HCHDPW_OffProtect.restype = ctypes.c_int64
    HCHDPW_OffProtect.argtypes = []
    
    ret = HCHDPW_OffProtect()
    return ret


# Int64  __stdcall HCHDVTPP_OnProtect(Int64 hwnd);
def HD盾_开启PPEx(窗口句柄: int) -> int:
    """
    保护窗口和进程(组件8)

    Args:
        窗口句柄: 目标窗口句柄，一般为父窗口句柄

    Returns:
        int: 返回值代码（查看HD返回值表）

    Note:
    需要调用HCHD_LoadDrv2传递参数8来安装保护盾HDPPEx驱动
    可以取消调用过HCHDVTPP_OffProtect所有保护的窗口句柄
    """
    hd = Config.get_hd()

    HCHDVTPP_OnProtect = hd.HCHDVTPP_OnProtect
    HCHDVTPP_OnProtect.restype = ctypes.c_int64
    HCHDVTPP_OnProtect.argtypes = [
        ctypes.c_int64
    ]

    ret = HCHDVTPP_OnProtect(ctypes.c_int64(窗口句柄))
    return ret

# Int64  __stdcall HCHDVTPP_OffProtect();
def HD盾_关闭PPEx(窗口句柄: int) -> int:
    """
    取消所有保护窗口和进程(组件8)

    Returns:
        int: 返回值代码（查看HD返回值表）

    Note:
    需要调用HCHD_LoadDrv2传递参数8来安装保护盾HDPPEx驱动
    可以取消调用过HCHDVTPP_OffProtect所有保护的窗口句柄
    """
    hd = Config.get_hd()

    HCHDVTPP_OffProtect = hd.HCHDVTPP_OffProtect
    HCHDVTPP_OffProtect.restype = ctypes.c_int64
    HCHDVTPP_OffProtect.argtypes = []

    ret = HCHDVTPP_OffProtect()
    return ret


# Int64  __stdcall HCHDVTPP_OnCapture(Int64 hwnd, int type);
def HD盾_反截图PPEx(窗口句柄: int,类型: int) -> int:
    """
    反截图(组件8)

    Args:
        窗口句柄: 目标窗口句柄，一般为父窗口句柄
        type:0关闭 1开启反截图模式1 2开启反截图模式2
    Returns:
        int: 返回值代码（查看HD返回值表）

    Note:
    需要调用HCHD_LoadDrv2传递参数8来安装保护盾HDPPEx驱动
    """
    hd = Config.get_hd()

    HCHDVTPP_OnCapture = hd.HCHDVTPP_OnCapture
    HCHDVTPP_OnCapture.restype = ctypes.c_int64
    HCHDVTPP_OnCapture.argtypes = [
        ctypes.c_int64,
        ctypes.c_int32
    ]

    ret = HCHDVTPP_OnCapture(ctypes.c_int64(窗口句柄),ctypes.c_int32(类型))
    return ret

# Int64  __stdcall HCHDVTPP_ReadBytes(Int64 pid, Int64 targetAddr, __int32 readSize, Int64 buffer, int type);
def HD盾_读字节集PPEx(pid: int,目标地址: int,读大小: int,缓冲区: int,类型) -> int:
    """
    读字节集(组件8)

    Args:
        pid:窗口PID
        targetAddr:目标地址
        readSize:读大小
        buffer:存读数据的缓冲区 注意:缓冲区大小一定要大于等于 读大小 最好相等
        type: 1.普通读写1 2.加强模式2  3.物理读写3
    Returns:
        int: 返回值代码（查看HD返回值表）

    Note:
    需要调用HCHD_LoadDrv2传递参数8来安装保护盾HDPPEx驱动
    """
    hd = Config.get_hd()

    HCHDVTPP_ReadBytes = hd.HCHDVTPP_ReadBytes
    HCHDVTPP_ReadBytes.restype = ctypes.c_int64
    HCHDVTPP_ReadBytes.argtypes = [
        ctypes.c_int64,
        ctypes.c_int64,
        ctypes.c_int32,
        ctypes.c_int64,
        ctypes.c_int32
    ]

    ret = HCHDVTPP_ReadBytes(ctypes.c_int64(pid),ctypes.c_int64(目标地址),ctypes.c_int32(读大小),ctypes.c_int64(缓冲区),ctypes.c_int32(类型))
    return ret


# Int64  __stdcall HCHDVTPP_WriteBytes(__int64 pid, __int64 targetAddr, __int32 writeSize, __int64 buffer, int type);
def HD盾_写字节集PPEx(pid: int,目标地址: int,写大小: int,缓冲区: int,类型) -> int:
    """
    读字节集(组件8)

    Args:
        pid:窗口PID
        targetAddr:目标地址
        writeSize:写大小
        buffer:存写数据的缓冲区
        type: 1 普通读写1 2加强模式2 3物理读写3
    Returns:
        int: 返回值代码（查看HD返回值表）

    Note:
    需要调用HCHD_LoadDrv2传递参数8来安装保护盾HDPPEx驱动
    """
    hd = Config.get_hd()

    HCHDVTPP_WriteBytes = hd.HCHDVTPP_WriteBytes
    HCHDVTPP_WriteBytes.restype = ctypes.c_int64
    HCHDVTPP_WriteBytes.argtypes = [
        ctypes.c_int64,
        ctypes.c_int64,
        ctypes.c_int32,
        ctypes.c_int64,
        ctypes.c_int32
    ]

    ret = HCHDVTPP_WriteBytes(ctypes.c_int64(pid),ctypes.c_int64(目标地址),ctypes.c_int32(写大小),ctypes.c_int64(缓冲区),ctypes.c_int32(类型))
    return ret


from .config import Config, ctypes
from .config import auto_encode

# INT64 __stdcall HCNT_GetWindowProcessId(INT64 hwnd);
def HDNT_窗口句柄取PID(窗口句柄: int) -> int:
    """
    获取目标窗口句柄的进程ID（R3层内核版本）
    
    Args:
        窗口句柄: 目标窗口的句柄值
    
    Returns:
        int: 进程ID（查看HD返回值表）
    
    Note:
        可以绕过一些检测，比如NP等
    """
    hd = Config.get_hd()
    
    HCNT_GetWindowProcessId = hd.HCNT_GetWindowProcessId
    HCNT_GetWindowProcessId.restype = ctypes.c_int64
    HCNT_GetWindowProcessId.argtypes = [ctypes.c_int64]
    
    ret = HCNT_GetWindowProcessId(ctypes.c_int64(窗口句柄))
    return ret


from .config import Config, ctypes
from .config import auto_encode

# INT64 __stdcall HCInject_Init(int 窗口序号, BOOL 是否重置);
def HD通讯_初始化(窗口序号: int, 是否重置: bool = False) -> int:
    """
    初始化插件环境
    
    Args:
        窗口序号: 窗口序号，从1开始
        是否重置: 是否重置，默认为False
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCInject_Init = hd.HCInject_Init
    HCInject_Init.restype = ctypes.c_int64
    HCInject_Init.argtypes = [
        ctypes.c_int32,
        ctypes.c_bool
    ]
    
    ret = HCInject_Init(ctypes.c_int32(窗口序号), ctypes.c_bool(是否重置))
    return ret

# INT64 __stdcall HCInject_InitEx(int 窗口序号, char* 预加载LUA模块全路径, BOOL 是否重置);
def HD通讯_初始化Ex(窗口序号: int, 预加载LUA模块全路径: str, 是否重置: bool = False) -> int:
    """
    初始化插件环境扩展版本
    
    Args:
        窗口序号: 窗口序号，从1开始
        预加载LUA模块全路径: 指定一个LUA文件路径+文件名，当插件被加载的时候会自动执行这个LUA文件，通讯后执行
        是否重置: 是否重置
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCInject_InitEx = hd.HCInject_InitEx
    HCInject_InitEx.restype = ctypes.c_int64
    HCInject_InitEx.argtypes = [
        ctypes.c_int32,
        ctypes.c_char_p,
        ctypes.c_bool
    ]
    
    ret = HCInject_InitEx(
        ctypes.c_int32(窗口序号), 
        auto_encode(预加载LUA模块全路径), 
        ctypes.c_bool(是否重置)
    )
    return ret

# INT64 __stdcall HCInject_InitExx(int 窗口序号, char* 预加载DLL模块全路径, char* 预加载LUA模块全路径, int addType, BOOL 是否重置);
def HD通讯_初始化Exx(窗口序号: int, 预加载DLL模块全路径: str = None, 预加载LUA模块全路径: str = None, 添加类型: int = -1, 是否重置: bool = False) -> int:
    """
    初始化插件环境扩展扩展版本
    
    Args:
        窗口序号: 窗口序号，从1开始
        预加载DLL模块全路径: 指定一个预加载的DLL，通讯前直接加载
        预加载LUA模块全路径: 指定一个LUA文件路径+文件名，当插件被加载的时候会自动执行这个LUA文件，通讯后执行
        添加类型: -1/0/1无痕无模块加载DLL，2普通加载DLL
        是否重置: 是否重置
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCInject_InitExx = hd.HCInject_InitExx
    HCInject_InitExx.restype = ctypes.c_int64
    HCInject_InitExx.argtypes = [
        ctypes.c_int32,
        ctypes.c_char_p,
        ctypes.c_char_p,
        ctypes.c_int32,
        ctypes.c_bool
    ]
    
    dll_path = auto_encode(预加载DLL模块全路径) if 预加载DLL模块全路径 else None
    lua_path = auto_encode(预加载LUA模块全路径) if 预加载LUA模块全路径 else None
    
    ret = HCInject_InitExx(
        ctypes.c_int32(窗口序号), 
        dll_path,
        lua_path,
        ctypes.c_int32(添加类型), 
        ctypes.c_bool(是否重置)
    )
    return ret

# INT64 __stdcall HCInject_GetPid(int 窗口序号);
def HD通讯_获取PID(窗口序号: int) -> int:
    """
    获取打开的进程PID
    
    Args:
        窗口序号: 窗口序号，从1开始
    
    Returns:
        int: 进程PID
    """
    hd = Config.get_hd()
    
    HCInject_GetPid = hd.HCInject_GetPid
    HCInject_GetPid.restype = ctypes.c_int64
    HCInject_GetPid.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCInject_GetPid(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCInject_GetPrePid(int 窗口序号);
def HD通讯_获取之前PID(窗口序号: int) -> int:
    """
    获取打开的进程之前的PID，可以用于重连
    
    Args:
        窗口序号: 窗口序号，从1开始
    
    Returns:
        int: 之前的进程PID
    """
    hd = Config.get_hd()
    
    HCInject_GetPrePid = hd.HCInject_GetPrePid
    HCInject_GetPrePid.restype = ctypes.c_int64
    HCInject_GetPrePid.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCInject_GetPrePid(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCInject_SetData(int 窗口序号, void* 用户数据缓冲区, int 数据大小);
def HD通讯_存数据(窗口序号: int, 用户数据缓冲区: bytes, 数据大小: int) -> int:
    """
    存自定义二进制数据
    
    Args:
        窗口序号: 窗口序号，从1开始
        用户数据缓冲区: 数据的来源缓冲区 (bytes) - 字节数组，如：b'\x01\x02\x03' 或 bytes([1, 2, 3])
        数据大小: 数据的来源缓冲区中有效数据的大小，总大小1024字节
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCInject_SetData = hd.HCInject_SetData
    HCInject_SetData.restype = ctypes.c_int64
    HCInject_SetData.argtypes = [
        ctypes.c_int32,
        ctypes.c_void_p,
        ctypes.c_int32
    ]
    
    用户数据缓冲区_ptr = ctypes.cast(用户数据缓冲区, ctypes.c_void_p).value if 用户数据缓冲区 else ctypes.c_void_p(0)
    
    ret = HCInject_SetData(
        ctypes.c_int32(窗口序号), 
        用户数据缓冲区_ptr, 
        ctypes.c_int32(数据大小)
    )
    return ret

# INT64 __stdcall HCInject_GetData(int 窗口序号, void* 用户数据缓冲区, int 缓冲区大小);
def HD通讯_取数据(窗口序号: int, 用户数据缓冲区, 缓冲区大小: int) -> int:
    """
    取自定义二进制数据
    
    Args:
        窗口序号: 窗口序号，从1开始
        用户数据缓冲区: 数据的存储缓冲区，取的数据就是用HCInject_SetData接口存的数据
        缓冲区大小: 数据的存储缓冲区数据的大小，不能小于等于0，从内部取的数据最大为1024
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCInject_GetData = hd.HCInject_GetData
    HCInject_GetData.restype = ctypes.c_int64
    HCInject_GetData.argtypes = [
        ctypes.c_int32,
        ctypes.c_void_p,
        ctypes.c_int32
    ]
    
    用户数据缓冲区_ptr = ctypes.cast(用户数据缓冲区, ctypes.c_void_p).value if 用户数据缓冲区 else ctypes.c_void_p(0)
    
    ret = HCInject_GetData(
        ctypes.c_int32(窗口序号), 
        用户数据缓冲区_ptr, 
        ctypes.c_int32(缓冲区大小)
    )
    return ret

# INT64 __stdcall HCInject_GetPreWinIndex(int pid);
def HD通讯_获取绑定窗口序号(进程PID: int) -> int:
    """
    通过进程PID尝试拿到绑定的窗口序号
    
    Args:
        进程PID: 指定一个进程PID
    
    Returns:
        int: 绑定的窗口序号
    """
    hd = Config.get_hd()
    
    HCInject_GetPreWinIndex = hd.HCInject_GetPreWinIndex
    HCInject_GetPreWinIndex.restype = ctypes.c_int64
    HCInject_GetPreWinIndex.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCInject_GetPreWinIndex(ctypes.c_int32(进程PID))
    return ret

# INT64 __stdcall HCInject_GetHwnd(int 窗口序号, char* className, char* title, int filter, BOOL bType, int mis);
def HD通讯_获取窗口句柄(窗口序号: int, 窗口类名: str, 窗口标题: str, 过滤器: int, 是否模糊匹配: bool, 循环毫秒: int) -> int:
    """
    获取窗口句柄，内部是通过PID获取窗口句柄，需要先安装好插件到目标进程
    
    Args:
        窗口序号: 窗口序号，从1开始
        窗口类名: 窗口类名，不指定为NULL/0
        窗口标题: 窗口标题，不指定为NULL/0
        过滤器: 1标题 2类名 8检测是否无父窗口 16检查窗口句柄是否有效，多个标识可以用|累加
        是否模糊匹配: True为模糊匹配，False为完全匹配
        循环毫秒: 循环检查毫秒数，<=0表示不需要循环获取，>0表示内部循环获取句柄直到有值
    
    Returns:
        int: 窗口句柄
    """
    hd = Config.get_hd()
    
    HCInject_GetHwnd = hd.HCInject_GetHwnd
    HCInject_GetHwnd.restype = ctypes.c_int64
    HCInject_GetHwnd.argtypes = [
        ctypes.c_int32,
        ctypes.c_char_p,
        ctypes.c_char_p,
        ctypes.c_int32,
        ctypes.c_bool,
        ctypes.c_int32
    ]
    
    ret = HCInject_GetHwnd(
        ctypes.c_int32(窗口序号), 
        auto_encode(窗口类名), 
        auto_encode(窗口标题), 
        ctypes.c_int32(过滤器), 
        ctypes.c_bool(是否模糊匹配), 
        ctypes.c_int32(循环毫秒)
    )
    return ret

# INT64 __stdcall HCInject_GetHwndEx(int 窗口序号);
def HD通讯_获取窗口句柄Ex(窗口序号: int) -> int:
    """
    获取窗口句柄扩展版本，需要内置，一般不需要调用这个获取句柄
    
    Args:
        窗口序号: 窗口序号，从1开始
    
    Returns:
        int: 窗口句柄
    """
    hd = Config.get_hd()
    
    HCInject_GetHwndEx = hd.HCInject_GetHwndEx
    HCInject_GetHwndEx.restype = ctypes.c_int64
    HCInject_GetHwndEx.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCInject_GetHwndEx(ctypes.c_int32(窗口序号))
    return ret

# INT64 __stdcall HCInject_Continue(int 窗口序号);
def HD通讯_继续(窗口序号: int) -> int:
    """
    继续环境加载操作
    
    Args:
        窗口序号: 窗口序号，从1开始
    
    Returns:
        int: 返回值代码
    """
    hd = Config.get_hd()
    
    HCInject_Continue = hd.HCInject_Continue
    HCInject_Continue.restype = ctypes.c_int64
    HCInject_Continue.argtypes = [
        ctypes.c_int32
    ]
    
    ret = HCInject_Continue(ctypes.c_int32(窗口序号))
    return ret
